import { ListOfPropertiesTemplate, PropertyTemplate } from "../../properties/factory/property-template";
import { AbstractProperty } from "../../properties/abstract-property";
import { PropertyId } from "../../properties/property-id";
import { ListOfProperties } from "../../properties/list-of-properties";
import { ListOfPropertiesImpl } from "../../properties/list-of-properties-impl";
import { ListIndex } from "../../properties/lists/index/list-index";
import { SiblingAccess } from "../../provider/list-provider/sibling-access";
import { AbstractDataProperty } from "../../properties/abstract-data-property";
import { Rule } from "../../rules/rule";
import { BackpressureConfig } from "../../properties/backpressure/backpressure-config";
import { PropertyConfig } from "./builder-options";
import { PropertyArrayListAsyncImpl, PropertyArrayListSyncImpl } from "../../properties/property-array-list-impl";
import { AsyncListProvider, ListProvider } from "../../provider/list-provider/list-provider";
import { PropertyArrayList, PropertyArrayListCrud, PropertyArrayListCrudAsync, PropertyArrayListReadonly, PropertyArrayListReadonlyAsync } from "../../properties/property-array-list";
import { ListOfPropertiesRuleBinding } from "./list-of-properties-rule-bindings";
import { PropertyArrayListRuleBinding } from "./property-array-list-rule-binding";
export declare enum SelectionMode {
    MultiSelect = 0,
    SingleSelect = 1
}
export declare class ListBuilder {
    private readonly listOfProperties;
    private readonly propertyList;
    private readonly asyncPropertyList;
    constructor(listOfProperties: <T extends AbstractDataProperty<D>, D>(id: string, itemTemplate: PropertyTemplate<T, D>, isMultiSelect: boolean) => ListOfPropertiesImpl<T, D>, propertyList: <T>(id: PropertyId, provider: ListProvider<T>, dependencies?: readonly AbstractProperty[], propertyConfig?: PropertyConfig & {
        backpressure?: BackpressureConfig;
    }, ownedProperties?: readonly AbstractProperty[]) => PropertyArrayListSyncImpl<T>, asyncPropertyList: <T>(id: PropertyId, provider: AsyncListProvider<T>, dependencies?: readonly AbstractProperty[], propertyConfig?: PropertyConfig & {
        backpressure?: BackpressureConfig;
    }, ownedProperties?: readonly AbstractProperty[]) => PropertyArrayListAsyncImpl<T>);
    create<T extends AbstractDataProperty<D>, D>(id: PropertyId, itemTemplate: PropertyTemplate<T, D>, selectionMode?: SelectionMode): ListOfProperties<T, D>;
    template<T extends AbstractDataProperty<D>, D>(id: string, factory: (listBuilder: ListBuilder, id: PropertyId, index?: ListIndex, siblingAccess?: SiblingAccess<ListOfProperties<T, D>>) => ListOfProperties<T, D>): ListOfPropertiesTemplate<T, D>;
    derived: {
        sync: <Dependencies extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies) => <T>(config: PropertyConfig & {
            derive: Rule<[...dependencies: Dependencies], T[]>;
        }) => PropertyArrayListReadonly<T>;
        async: <Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies_1) => <T_1>(config: PropertyConfig & {
            derive: Rule<[...dependencies: Dependencies_1], Promise<T_1[]>>;
        }) => PropertyArrayListReadonlyAsync<T_1>;
    };
    crud: {
        sync: <Dependencies extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies) => <T>(config?: (PropertyConfig & {
            resourceProvider?: Rule<[...dependencies: Dependencies], T[]> | undefined;
        }) | undefined) => PropertyArrayListCrud<T>;
        async: <Dependencies_1 extends readonly AbstractProperty[]>(id: PropertyId, ...dependencies: Dependencies_1) => <T_1>(config: PropertyConfig & {
            getElements: Rule<[...dependencies: Dependencies_1], Promise<T_1[]>>;
            addElement: (propertyData: T_1, index?: number | undefined) => Promise<void>;
            updateElement: (propertyData: T_1, index: number) => Promise<void>;
            removeElement: (index: number) => Promise<void>;
        }) => PropertyArrayListCrudAsync<T_1>;
    };
    bindListOfProperties<T extends AbstractDataProperty<D>, D>(list: ListOfProperties<T, D>): ListOfPropertiesRuleBinding<T, D>;
    bindPropertyArrayList<D>(list: PropertyArrayList<D>): PropertyArrayListRuleBinding<D>;
}
