"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ListBuilder = exports.SelectionMode = void 0;
const derived_list_provider_1 = require("../../provider/list-provider/derived-list-provider");
const derived_async_list_provider_1 = require("../../provider/list-provider/derived-async-list-provider");
const crud_async_list_provider_1 = require("../../provider/list-provider/crud-async-list-provider");
const crud_list_provider_1 = require("../../provider/list-provider/crud-list-provider");
const list_of_properties_rule_bindings_1 = require("./list-of-properties-rule-bindings");
const property_array_list_rule_binding_1 = require("./property-array-list-rule-binding");
var SelectionMode;
(function (SelectionMode) {
    SelectionMode[SelectionMode["MultiSelect"] = 0] = "MultiSelect";
    SelectionMode[SelectionMode["SingleSelect"] = 1] = "SingleSelect";
})(SelectionMode = exports.SelectionMode || (exports.SelectionMode = {}));
class ListBuilder {
    constructor(listOfProperties, propertyList, asyncPropertyList) {
        this.listOfProperties = listOfProperties;
        this.propertyList = propertyList;
        this.asyncPropertyList = asyncPropertyList;
        this.derived = {
            sync: (id, ...dependencies) => {
                return (config) => {
                    const provider = new derived_list_provider_1.DerivedListProvider(dependencies, (deps) => config.derive(...deps));
                    return this.propertyList(id, provider, dependencies, config);
                };
            },
            async: (id, ...dependencies) => {
                return (config) => {
                    const provider = new derived_async_list_provider_1.DerivedAsyncListProvider(dependencies, (deps) => config.derive(...deps));
                    return this.asyncPropertyList(id, provider, dependencies, config);
                };
            }
        };
        this.crud = {
            sync: (id, ...dependencies) => {
                return (config) => {
                    if (!config) {
                        config = {};
                    }
                    if (!config.resourceProvider) {
                        config.resourceProvider = () => [];
                    }
                    const provider = new crud_list_provider_1.CrudListProvider(dependencies, (deps) => config.resourceProvider(...deps));
                    return this.propertyList(id, provider, dependencies, config);
                };
            },
            async: (id, ...dependencies) => {
                return (config) => {
                    const provider = new crud_async_list_provider_1.CrudAsyncListProvider(dependencies, config.addElement, (deps) => config.getElements(...deps), config.updateElement, config.removeElement);
                    return this.asyncPropertyList(id, provider, dependencies, config);
                };
            }
        };
    }
    create(id, itemTemplate, selectionMode) {
        return this.listOfProperties(id, itemTemplate, selectionMode === SelectionMode.MultiSelect);
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    bindListOfProperties(list) {
        return new list_of_properties_rule_bindings_1.ListOfPropertiesRuleBinding(list);
    }
    bindPropertyArrayList(list) {
        return new property_array_list_rule_binding_1.PropertyArrayListRuleBinding(list);
    }
}
exports.ListBuilder = ListBuilder;
