import { DerivedListProvider } from "../../provider/list-provider/derived-list-provider";
import { DerivedAsyncListProvider } from "../../provider/list-provider/derived-async-list-provider";
import { CrudAsyncListProvider } from "../../provider/list-provider/crud-async-list-provider";
import { CrudListProvider } from "../../provider/list-provider/crud-list-provider";
import { ListOfPropertiesRuleBinding } from "./list-of-properties-rule-bindings";
import { PropertyArrayListRuleBinding } from "./property-array-list-rule-binding";
export var SelectionMode;
(function (SelectionMode) {
    SelectionMode[SelectionMode["MultiSelect"] = 0] = "MultiSelect";
    SelectionMode[SelectionMode["SingleSelect"] = 1] = "SingleSelect";
})(SelectionMode || (SelectionMode = {}));
export class ListBuilder {
    constructor(listOfProperties, propertyList, asyncPropertyList) {
        this.listOfProperties = listOfProperties;
        this.propertyList = propertyList;
        this.asyncPropertyList = asyncPropertyList;
        this.derived = {
            sync: (id, ...dependencies) => {
                return (config) => {
                    const provider = new DerivedListProvider(dependencies, (deps) => config.derive(...deps));
                    return this.propertyList(id, provider, dependencies, config);
                };
            },
            async: (id, ...dependencies) => {
                return (config) => {
                    const provider = new DerivedAsyncListProvider(dependencies, (deps) => config.derive(...deps));
                    return this.asyncPropertyList(id, provider, dependencies, config);
                };
            }
        };
        this.crud = {
            sync: (id, ...dependencies) => {
                return (config) => {
                    if (!config) {
                        config = {};
                    }
                    if (!config.resourceProvider) {
                        config.resourceProvider = () => [];
                    }
                    const provider = new CrudListProvider(dependencies, (deps) => config.resourceProvider(...deps));
                    return this.propertyList(id, provider, dependencies, config);
                };
            },
            async: (id, ...dependencies) => {
                return (config) => {
                    const provider = new CrudAsyncListProvider(dependencies, config.addElement, (deps) => config.getElements(...deps), config.updateElement, config.removeElement);
                    return this.asyncPropertyList(id, provider, dependencies, config);
                };
            }
        };
    }
    create(id, itemTemplate, selectionMode) {
        return this.listOfProperties(id, itemTemplate, selectionMode === SelectionMode.MultiSelect);
    }
    template(id, factory) {
        return (prefix, index, siblingAccess) => factory(this, `${prefix}_${id}`, index, siblingAccess);
    }
    bindListOfProperties(list) {
        return new ListOfPropertiesRuleBinding(list);
    }
    bindPropertyArrayList(list) {
        return new PropertyArrayListRuleBinding(list);
    }
}
